/*
 *  libpipi       Proper image processing implementation library
 *  Copyright (c) 2004-2008 Sam Hocevar <sam@zoy.org>
 *                All Rights Reserved
 *
 *  $Id$
 *
 *  This library is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

/*
 * floydsteinberg.c: Floyd-Steinberg dithering functions
 */

#include "config.h"
#include "common.h"

#include "pipi.h"
#include "pipi_internals.h"

pipi_image_t *pipi_floydsteinberg(pipi_image_t *src)
{
    pipi_image_t *dst;
    pipi_pixels_t *srcp, *dstp;
    float *srcdata, *dstdata;
    int x, y, w, h;

    w = src->w;
    h = src->h;

    srcp = pipi_getpixels(src, PIPI_PIXELS_Y_F);
    srcdata = (float *)srcp->pixels;

    dst = pipi_new(w, h);
    dstp = pipi_getpixels(dst, PIPI_PIXELS_Y_F);
    dstdata = (float *)dstp->pixels;

    for(y = 0; y < h; y++)
    {
        for(x = 0; x < w; x++)
        {
            float p, q, e;

            p = srcdata[y * w + x];
            q = p < 0.5 ? 0. : 1.;
            dstdata[y * w + x] = q;

            e = p - q;
            if(x < w - 1)
                srcdata[y * w + x + 1] += e * .4375;
            if(y < h - 1)
            {
                if(x > 0)
                    srcdata[(y + 1) * w + x - 1] += e * .1875;
                srcdata[(y + 1) * w + x] += e * .3125;
                if(x < w - 1)
                    srcdata[(y + 1) * w + x + 1] += e * .0625;
            }
        }
    }

    return dst;
}

